from .base import ValidationError


class EnumValidator(object):
    def __init__(self, *choices):
        self.choices = choices

    def validate(self, field, data):
        if data not in self.choices:
            raise ValidationError("{} must be one of {}.  Got {}.".format(field.name, self.choices, data))



import datetime
import uuid

from .base import ValidationError, InvalidTypeValidationError, BaseField

__all__ = ("Field", "StringField", "BooleanField", "IntegerField", "FloatField",
           "ObjectField", "ListField", "DateTimeField", "DateField", "TimeField", "UuidField")

Field = BaseField


class BaseTypeValidatorField(object):
    basetypes = ()

    def clean(self, data):
        if not isinstance(data, self.basetypes):
            raise InvalidTypeValidationError(self.name, str(self.basetypes), type(data))
        return data

    def object_to_data(self, obj):
        if not isinstance(obj, self.basetypes):
            raise InvalidTypeValidationError(self.object_field_name, str(self.basetypes), type(obj))
        return obj


class StringField(BaseTypeValidatorField, BaseField):
    """
    Represents a string field.
    """
    try:
        basetypes = (basestring,)
    except NameError:
        basetypes = (str,)


class BooleanField(BaseTypeValidatorField, BaseField):
    """
    Represents a boolean field.
    """
    basetypes = (bool,)


class IntegerField(BaseTypeValidatorField, BaseField):
    """
    Represents an integer field.  In python 2.7, this could be either an int or a long.  In python 3, it is just simply
    an 'int'.
    """
    basetypes = (int,)

    try:
        basetypes += (long,)
    except NameError:
        pass


class FloatField(BaseTypeValidatorField, BaseField):
    """
    Represents a floating point field.
    """
    basetypes = (float,)


class ObjectField(BaseField):
    """
    Represents an more complex object field.

    Specify the serializer to user as the argument to serialzer_class.
    """
    def __init__(self, serializer_class, *args, **kwargs):
        super(ObjectField, self).__init__(*args, **kwargs)
        self.serializer_class = serializer_class

    def clean(self, data):
        s = self.serializer_class(data=data)
        s.validate()
        return s.object

    def object_to_data(self, obj):
        s = self.serializer_class(object=obj)
        s.validate()
        return s.data


class ListField(BaseField):
    """
    Represents a list of objects of varying types.

    :param allowed_types: is either a list or tuple of Field types that are allowed in the list.  If just a field is
                          provided, then it is the only type allowed.
    """
    def __init__(self, allowed_types, *args, **kwargs):
        super(ListField, self).__init__(*args, **kwargs)
        if isinstance(allowed_types, BaseField):
            allowed_types = (allowed_types,)
        self.allowed_types = tuple(allowed_types)

    def clean(self, data):
        if not isinstance(data, list):
            raise InvalidTypeValidationError(self.name, "list", type(data))

        res = []
        for item_i, item in enumerate(data):
            for allowed_type in self.allowed_types:
                try:
                    obj = allowed_type.clean(item)
                except ValidationError:
                    pass
                else:
                    res.append(obj)
                    break
            else:
                raise InvalidTypeValidationError(
                    "{}[{}]".format(self.name, item_i), str(self.allowed_types), type(item)
                )

        return res

    def object_to_data(self, obj):
        res = []
        for item_i, item in enumerate(obj):
            for allowed_type in self.allowed_types:
                try:
                    data = allowed_type.object_to_data(item)
                except ValidationError:
                    pass
                else:
                    res.append(data)
                    break
            else:
                raise InvalidTypeValidationError(
                    "{}[{}]".format(self.name, item_i), str(self.allowed_types), type(item)
                )
        return res


class DateTimeField(StringField):
    """
    Represents a datetime object.

    Pass in a format string as the fmt parameter to set the format string, or you can pass in a callable using the
    'parse' keyword.
    """
    default_fmt = "%Y-%m-%d %H:%M:%S.%f"
    instance_type = datetime.datetime

    def __init__(self, fmt=None, parse=None, *args, **kwargs):
        super(DateTimeField, self).__init__(*args, **kwargs)
        self.fmt = fmt or self.default_fmt
        self.parse = parse or (lambda s: datetime.datetime.strptime(s, self.fmt))

    def clean(self, data):
        data = super(DateTimeField, self).clean(data)

        try:
            res = self.parse(data)
        except ValueError as ex:
            raise ValidationError(str(ex))
        else:
            return res

    def object_to_data(self, obj):
        if not isinstance(obj, self.instance_type):
            raise InvalidTypeValidationError(self.name, "datetime", type(obj))
        return obj.strftime(self.fmt)


class DateField(DateTimeField):
    """
    Represents a date object.
    """

    default_fmt = "%Y-%m-%d"
    instance_type = datetime.date

    def clean(self, data):
        return super(DateField, self).clean(data).date()


class TimeField(DateTimeField):
    """
    Represents a time object.
    """

    default_fmt = "%H:%M:%S.%f"
    instance_type = datetime.time

    def clean(self, data):
        return super(TimeField, self).clean(data).time()


class UuidField(StringField):
    """
    Represents a UUID.
    """

    def clean(self, data):
        data = super(UuidField, self).clean(data)

        try:
            res = uuid.UUID(data)
        except ValueError as ex:
            raise ValidationError(str(ex))
        else:
            return res

    def object_to_data(self, obj):
        if not isinstance(obj, uuid.UUID):
            raise InvalidTypeValidationError(self.name, "uuid", type(obj))
        return str(obj)



from . import fields
from . import base
from . import validators

from .base import (ValidationError, InvalidTypeValidationError, Serializer)

__all__ = ("fields", "base", "validators", "ValidationError",
           "InvalidTypeValidationError", "Serializer")



# Copied from the 'six' module.
def with_metaclass(meta, *bases):
    """Create a base class with a metaclass."""
    class metaclass(meta):
        def __new__(cls, name, this_bases, d):
            return meta(name, bases, d)
    return type.__new__(metaclass, 'temporary_class', (), {})


try:
    basestring
except NameError:
    basestring = str


class ValidationError(Exception):
    def __init__(self, errors):
        if isinstance(errors, basestring):
            errors = [errors]
        self.errors = errors
        super(ValidationError, self).__init__(str(self.errors))


class InvalidTypeValidationError(ValidationError):
    def __init__(self, field_name, expected, got):
        super(InvalidTypeValidationError, self).__init__("{} must be a {}.  Got {}.".format(field_name, expected, got))


class BaseField(object):
    def __init__(self, name=None, required=False, allow_null=True, validators=None, hidden=False, readonly=False):
        self.name = name
        self.object_field_name = name
        self.required = required
        self.allow_null = allow_null
        self.parent = None
        self.validators = validators or []
        self.hidden = hidden
        self.readonly = readonly

    def clean(self, data):
        return data

    def base_clean(self, data):
        if data is None:
            if not self.allow_null:
                raise ValidationError("{}/{} cannot be null/None".format(self.name, self.object_field_name))
            return None
        data = self.clean(data)
        for validator in self.validators:
            validator.validate(self, data)
        return data

    def object_to_data(self, obj):
        return obj

    def base_object_to_data(self, obj):
        if obj is None:
            if not self.allow_null:
                raise ValidationError("{}/{} cannot be null/None".format(self.name, self.object_field_name))
            return None
        return self.object_to_data(obj)


class DefaultMeta(object):
    pass


class DefaultModel(object):
    pass


class SerializerMetaclass(type):
    def __new__(cls, name, bases, attrs):
        options = DefaultMeta

        if "Meta" in attrs:
            options = attrs.pop("Meta")

        fields = []
        for k, v in attrs.items():
            if isinstance(v, BaseField):
                if v.name is None:
                    v.name = k
                v.object_field_name = k
                fields.append(v)

        new_class_attrs = {k: v for k, v in attrs.items() if not isinstance(v, BaseField)}
        new_class_attrs["fields"] = fields
        new_class_attrs["options"] = options
        ret = super(SerializerMetaclass, cls).__new__(cls, name, bases, new_class_attrs)
        for field in fields:
            field.parent = ret
        return ret


class BaseSerializer(object):
    fields = []
    options = None

    def __init__(self):
        pass

    @classmethod
    def load(self, data):
        model_class = getattr(self.options, "model", DefaultModel)
        model_class_args = getattr(self.options, "model_init_args", ())
        model_class_kwargs = getattr(self.options, "model_init_kwargs", {})

        obj = model_class(*model_class_args, **model_class_kwargs)
        self.update(data, obj)
        return obj

    @classmethod
    def update(self, data, obj):
        errors = []
        for field in self.fields:
            if field.required and field.name not in data:
                errors.append("Field {} is missing.".format(field.name))

            if field.readonly and field.name in data:
                errors.append("Field {} is read only.".format(field.name))

        if errors:
            raise ValidationError(errors)

        for field in self.fields:
            try:
                field_obj = field.base_clean(data[field.name])
            except ValidationError as ex:
                errors.extend(ex.errors)
            except KeyError:
                pass
            else:
                setattr(obj, field.object_field_name, field_obj)

        if errors:
            raise ValidationError(errors)

    @classmethod
    def dump(self, obj):
        errors = []
        for field in self.fields:
            if field.required and not hasattr(self.object, field.object_field_name):
                errors.append("Field {} is missing from object.".format(field.object_field_name))

        if errors:
            raise ValidationError(errors)

        data = {}
        for field in self.fields:
            if not field.hidden:
                try:
                    field_data = field.base_object_to_data(getattr(obj, field.object_field_name))
                except ValidationError as ex:
                    errors.extend(ex.errors)
                else:
                    data[field.name] = field_data

        if errors:
            raise ValidationError(errors)

        return data


class Serializer(with_metaclass(SerializerMetaclass, BaseSerializer)):
    """
    Base class for a new serializer.
    """
    pass



"""
Package to convert arbitrary python objects into DTOs ready
for serialization and validation.
"""

__version__ = "0.1.3"

from setuptools import setup


setup(
    name="beerializer",
    author="Songbee Team",
    author_email="hi@songbee.net",
    url="https://github.com/Songbee/beerializer",
    version=__version__,
    description=__doc__.replace("\n", " "),
    long_description=open("README.rst").read(),
    keywords="dto serializer serialize REST marshal JSON",
    packages=["beerializer"],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: Public Domain",
        "Programming Language :: Python",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content :: CGI Tools/Libraries",
        "Topic :: Software Development",
    ]
)
